@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2016 Hive Solutions Lda.
+# Copyright (c) 2008-2017 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2016 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -39,6 +39,9 @@ __license__ = "Apache License, Version 2.0"
 
 import threading
 
+from . import config
+from . import exceptions
+
 class AsyncManager(object):
 
     def __init__(self, owner):
@@ -68,10 +71,13 @@ class SimpleManager(AsyncManager):
 
 class QueueManager(AsyncManager):
 
-    def start(self):
-        self.thread = threading.Thread(target = self.handler)
+    def __init__(self, owner):
+        AsyncManager.__init__(self, owner)
         self.queue = []
         self.condition = threading.Condition()
+
+    def start(self):
+        self.thread = threading.Thread(target = self.handler)
         self.running = True
         self.thread.daemon = True
         self.thread.start()
@@ -103,3 +109,46 @@ class QueueManager(AsyncManager):
                     exception,
                     message = "Problem handling async item: %s"
                 )
+
+def unavailable(*args, **kwargs):
+    raise exceptions.AppierException(
+        message = "No support for async available"
+    )
+
+# determines the target service configuration that is
+# going to be used, this is going to be used to create
+# the proper adaptation for the async models
+server = config.conf("SERVER", None)
+if server == "netius":
+    import netius
+    Future = netius.Future
+    ensure_async = netius.ensure
+    coroutine = netius.coroutine
+    sleep = netius.sleep
+    wait = netius.wait
+    notify = netius.notify
+else:
+    Future = unavailable
+    ensure_async = unavailable
+    coroutine = unavailable
+    sleep = unavailable
+    wait = unavailable
+    notify = unavailable
+
+def ensure_a(*args, **kwargs):
+    yield ensure_async(*args, **kwargs)
+
+def header_a():
+    yield -1
+
+def to_coroutine(callable, *args, **kwargs):
+    future = kwargs.pop("future", None) or Future()
+    callback = kwargs.get("callback", None)
+
+    def callback_wrap(result, *args, **kwargs):
+        future.set_result(result)
+        callback and callback(result, *args, **kwargs)
+
+    kwargs["callback"] = callback_wrap
+    callable(*args, **kwargs)
+    yield future
