@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2016 Hive Solutions Lda.
+# Copyright (c) 2008-2017 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2016 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -125,10 +125,20 @@ def conf_prefix(prefix):
         configs[name] = value
     return configs
 
+def conf_suffix(suffix):
+    config = dict()
+    for name, value in CONFIGS.items():
+        if not name.endswith(suffix): continue
+        config[name] = value
+    return config
+
 def conf_s(name, value):
     global CONFIGS
     CONFIGS[name] = value
 
+def conf_d():
+    return CONFIGS
+
 def load(names = (FILE_NAME,), path = None, encoding = "utf-8"):
     paths = []
     homes = get_homes()
@@ -223,4 +233,21 @@ def get_homes(
 
     return HOMES
 
+def _is_devel():
+    """
+    Simple debug/development level detection mechanism to be
+    used at load time to determine if the system is running
+    under a development (debug) environment.
+
+    This function should not be used at runtime as there are
+    other (more powerful) mechanisms to archive the same
+    type of results.
+
+    :rtype: bool
+    :return: If the current environment is running under a
+    development type level of traceability.
+    """
+
+    return conf("LEVEL", "INFO") in ("DEBUG",)
+
 load()
