@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2016 Hive Solutions Lda.
+# Copyright (c) 2008-2017 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2016 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -65,6 +65,9 @@ class DataAdapter(object):
     def collection(self, name, *args, **kwargs):
         raise exceptions.NotImplementedError()
 
+    def reset(self):
+        raise exceptions.NotImplementedError()
+
     def get_db(self):
         raise exceptions.NotImplementedError()
 
@@ -112,6 +115,9 @@ class MongoAdapter(DataAdapter):
         collection = db[name]
         return MongoCollection(self, name, collection)
 
+    def reset(self):
+        return mongo.reset_connection()
+
     def get_db(self):
         return mongo.get_db()
 
@@ -139,6 +145,9 @@ class TinyAdapter(DataAdapter):
         table = db.table(name)
         return TinyCollection(self, name, table)
 
+    def reset(self):
+        pass
+
     def get_db(self):
         if not self._db == None: return self._db
         method = getattr(self, "_get_db_%s" % self.storage)
