@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2016 Hive Solutions Lda.
+# Copyright (c) 2008-2017 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2016 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -45,6 +45,7 @@ import logging
 import threading
 
 from . import util
+from . import async
 from . import common
 from . import legacy
 from . import typesf
@@ -69,6 +70,10 @@ AUTH_ERRORS = (401, 403, 440, 499)
 considered to be authentication related and for which a
 new authentication try will be performed """
 
+ACCESS_LOCK = threading.RLock()
+""" Global access lock used for locking global operations
+that required thread safety under the http infra-structure """
+
 def try_auth(auth_callback, params, headers = None):
     if not auth_callback: raise
     if headers == None: headers = dict()
@@ -89,8 +94,11 @@ def get(
     params = None,
     headers = None,
     handle = None,
+    silent = None,
     redirect = None,
-    auth_callback = None
+    timeout = None,
+    auth_callback = None,
+    **kwargs
 ):
     return _method(
         _get,
@@ -98,8 +106,11 @@ def get(
         params = params,
         headers = headers,
         handle = handle,
+        silent = silent,
         redirect = redirect,
-        auth_callback = auth_callback
+        timeout = timeout,
+        auth_callback = auth_callback,
+        **kwargs
     )
 
 def post(
@@ -111,8 +122,11 @@ def post(
     headers = None,
     mime = None,
     handle = None,
+    silent = None,
     redirect = None,
-    auth_callback = None
+    timeout = None,
+    auth_callback = None,
+    **kwargs
 ):
     return _method(
         _post,
@@ -124,8 +138,11 @@ def post(
         headers = headers,
         mime = mime,
         handle = handle,
+        silent = silent,
         redirect = redirect,
-        auth_callback = auth_callback
+        timeout = timeout,
+        auth_callback = auth_callback,
+        **kwargs
     )
 
 def put(
@@ -137,8 +154,11 @@ def put(
     headers = None,
     mime = None,
     handle = None,
+    silent = None,
     redirect = None,
-    auth_callback = None
+    timeout = None,
+    auth_callback = None,
+    **kwargs
 ):
     return _method(
         _put,
@@ -150,8 +170,11 @@ def put(
         headers = headers,
         mime = mime,
         handle = handle,
+        silent = silent,
         redirect = redirect,
-        auth_callback = auth_callback
+        timeout = timeout,
+        auth_callback = auth_callback,
+        **kwargs
     )
 
 def delete(
@@ -159,8 +182,11 @@ def delete(
     params = None,
     headers = None,
     handle = None,
+    silent = None,
     redirect = None,
-    auth_callback = None
+    timeout = None,
+    auth_callback = None,
+    **kwargs
 ):
     return _method(
         _delete,
@@ -168,8 +194,11 @@ def delete(
         params = params,
         headers = headers,
         handle = handle,
+        silent = silent,
         redirect = redirect,
-        auth_callback = auth_callback
+        timeout = timeout,
+        auth_callback = auth_callback,
+        **kwargs
     )
 
 def patch(
@@ -181,8 +210,11 @@ def patch(
     headers = None,
     mime = None,
     handle = None,
+    silent = None,
     redirect = None,
-    auth_callback = None
+    timeout = None,
+    auth_callback = None,
+    **kwargs
 ):
     return _method(
         _patch,
@@ -194,10 +226,33 @@ def patch(
         headers = headers,
         mime = mime,
         handle = handle,
+        silent = silent,
         redirect = redirect,
-        auth_callback = auth_callback
+        timeout = timeout,
+        auth_callback = auth_callback,
+        **kwargs
     )
 
+def get_a(*args, **kwargs):
+    kwargs["async"] = True
+    for value in async.to_coroutine(get, *args, **kwargs): yield value
+
+def post_a(*args, **kwargs):
+    kwargs["async"] = True
+    for value in async.to_coroutine(post, *args, **kwargs): yield value
+
+def put_a(*args, **kwargs):
+    kwargs["async"] = True
+    for value in async.to_coroutine(put, *args, **kwargs): yield value
+
+def delete_a(*args, **kwargs):
+    kwargs["async"] = True
+    for value in async.to_coroutine(delete, *args, **kwargs): yield value
+
+def patch_a(*args, **kwargs):
+    kwargs["async"] = True
+    for value in async.to_coroutine(patch, *args, **kwargs): yield value
+
 def _method(method, *args, **kwargs):
     try:
         auth_callback = kwargs.pop("auth_callback", None)
@@ -220,7 +275,10 @@ def _get(
     params = None,
     headers = None,
     handle = None,
-    redirect = None
+    silent = None,
+    redirect = None,
+    timeout = None,
+    **kwargs
 ):
     return _method_empty(
         "GET",
@@ -228,7 +286,10 @@ def _get(
         params = params,
         headers = headers,
         handle = handle,
-        redirect = redirect
+        silent = silent,
+        redirect = redirect,
+        timeout = timeout,
+        **kwargs
     )
 
 def _post(
@@ -240,7 +301,10 @@ def _post(
     headers = None,
     mime = None,
     handle = None,
-    redirect = None
+    silent = None,
+    redirect = None,
+    timeout = None,
+    **kwargs
 ):
     return _method_payload(
         "POST",
@@ -252,7 +316,10 @@ def _post(
         headers = headers,
         mime = mime,
         handle = handle,
-        redirect = redirect
+        silent = silent,
+        redirect = redirect,
+        timeout = timeout,
+        **kwargs
     )
 
 def _put(
@@ -264,7 +331,10 @@ def _put(
     headers = None,
     mime = None,
     handle = None,
-    redirect = None
+    silent = None,
+    redirect = None,
+    timeout = None,
+    **kwargs
 ):
     return _method_payload(
         "PUT",
@@ -276,7 +346,10 @@ def _put(
         headers = headers,
         mime = mime,
         handle = handle,
-        redirect = redirect
+        silent = silent,
+        redirect = redirect,
+        timeout = timeout,
+        **kwargs
     )
 
 def _delete(
@@ -284,7 +357,10 @@ def _delete(
     params = None,
     headers = None,
     handle = None,
-    redirect = None
+    silent = None,
+    redirect = None,
+    timeout = None,
+    **kwargs
 ):
     return _method_empty(
         "DELETE",
@@ -292,7 +368,10 @@ def _delete(
         params = params,
         headers = headers,
         handle = handle,
-        redirect = redirect
+        silent = silent,
+        redirect = redirect,
+        timeout = timeout,
+        **kwargs
     )
 
 def _patch(
@@ -304,7 +383,10 @@ def _patch(
     headers = None,
     mime = None,
     handle = None,
-    redirect = None
+    silent = None,
+    redirect = None,
+    timeout = None,
+    **kwargs
 ):
     return _method_payload(
         "PATCH",
@@ -316,7 +398,10 @@ def _patch(
         headers = headers,
         mime = mime,
         handle = handle,
-        redirect = redirect
+        silent = silent,
+        redirect = redirect,
+        timeout = timeout,
+        **kwargs
     )
 
 def _method_empty(
@@ -325,26 +410,34 @@ def _method_empty(
     params = None,
     headers = None,
     handle = None,
+    silent = None,
     redirect = None,
-    timeout = TIMEOUT
+    timeout = None,
+    **kwargs
 ):
     if handle == None: handle = False
-    if redirect == None: redirect = False
+    if silent == None: silent = config.conf("HTTP_SILENT", False, cast = bool)
+    if redirect == None: redirect = config.conf("HTTP_REDIRECT", False, cast = bool)
+    if timeout == None: timeout = config.conf("HTTP_TIMEOUT", TIMEOUT, cast = int)
     values = params or dict()
 
-    logging.info("%s %s with '%s'" % (name, url, str(values)))
+    values_s = " with '%s'" % str(values) if values else ""
+    if not silent: logging.debug("%s %s%s" % (name, url, values_s))
 
     url, scheme, host, authorization, extra = _parse_url(url)
     if extra: values.update(extra)
     data = _urlencode(values)
 
-    headers = headers or dict()
+    headers = dict(headers) if headers else dict()
     if host: headers["Host"] = host
     if authorization: headers["Authorization"] = "Basic %s" % authorization
     url = url + "?" + data if data else url
     url = str(url)
 
-    file = _resolve(url, name, headers, None, timeout)
+    _method_callback(handle, kwargs)
+    file = _resolve(url, name, headers, None, silent, timeout, **kwargs)
+    if file == None: return file
+
     try: result = file.read()
     finally: file.close()
 
@@ -352,9 +445,17 @@ def _method_empty(
     info = file.info()
 
     location = info.get("Location", None) if redirect else None
-    if location: return _redirect(location, scheme, host, handle, redirect)
+    if location: return _redirect(
+        location,
+        scheme,
+        host,
+        handle = handle,
+        silent = silent,
+        redirect = redirect,
+        timeout = timeout
+    )
 
-    logging.info("%s %s returned '%d'" % (name, url, code))
+    if not silent: logging.debug("%s %s returned '%d'" % (name, url, code))
 
     result = _result(result, info)
     return (result, file) if handle else result
@@ -369,14 +470,19 @@ def _method_payload(
     headers = None,
     mime = None,
     handle = None,
+    silent = None,
     redirect = None,
-    timeout = TIMEOUT
+    timeout = None,
+    **kwargs
 ):
     if handle == None: handle = False
-    if redirect == None: redirect = False
+    if silent == None: silent = config.conf("HTTP_SILENT", False, cast = bool)
+    if redirect == None: redirect = config.conf("HTTP_REDIRECT", False, cast = bool)
+    if timeout == None: timeout = config.conf("HTTP_TIMEOUT", TIMEOUT, cast = int)
     values = params or dict()
 
-    logging.info("%s %s with '%s'" % (name, url, str(params)))
+    values_s = " with '%s'" % str(values) if values else ""
+    if not silent: logging.debug("%s %s%s" % (name, url, values_s))
 
     url, scheme, host, authorization, extra = _parse_url(url)
     if extra: values.update(extra)
@@ -401,14 +507,17 @@ def _method_payload(
     data = legacy.bytes(data)
     length = len(data) if data else 0
 
-    headers = headers or dict()
-    headers["Content-Length"] = length
+    headers = dict(headers) if headers else dict()
+    headers["Content-Length"] = str(length)
     if mime: headers["Content-Type"] = mime
     if host: headers["Host"] = host
     if authorization: headers["Authorization"] = "Basic %s" % authorization
     url = str(url)
 
-    file = _resolve(url, name, headers, data, timeout)
+    _method_callback(handle, **kwargs)
+    file = _resolve(url, name, headers, data, silent, timeout, **kwargs)
+    if file == None: return file
+
     try: result = file.read()
     finally: file.close()
 
@@ -416,85 +525,239 @@ def _method_payload(
     info = file.info()
 
     location = info.get("Location", None) if redirect else None
-    if location: return _redirect(location, scheme, host, handle, redirect)
+    if location: return _redirect(
+        location,
+        scheme,
+        host,
+        handle = handle,
+        silent = silent,
+        redirect = redirect,
+        timeout = timeout
+    )
 
-    logging.info("%s %s returned '%d'" % (name, url, code))
+    if not silent: logging.debug("%s %s returned '%d'" % (name, url, code))
 
     result = _result(result, info)
     return (result, file) if handle else result
 
-def _redirect(location, scheme, host, handle, redirect):
+def _method_callback(handle, kwargs):
+    # tries to determine if a callback value has been registered
+    # in the set of keyword argument and if tat's not the case
+    # returns immediately (nothing to be done)
+    callback = kwargs.get("callback", None)
+    if not callback: return
+
+    def callback_wrap(file):
+        # determines if the received file is valid (no error)
+        # or if instead there's an error with the connection
+        # and an invalid/unset value has been provided
+        if file:
+            info = file.info()
+            try: result = file.read()
+            finally: file.close()
+            result = _result(result, info)
+        else:
+            result = None
+
+        # taking into account the handle flag determines the
+        # kind of result structure that should be "returned"
+        result = (result, file) if handle else (result,)
+        callback(*result)
+
+    # sets the "new" callback clojure in the set of keyword
+    # based arguments for the calling of the http handler method
+    # so that this new callback is called instead of the original
+    kwargs["callback"] = callback_wrap
+
+def _redirect(
+    location,
+    scheme,
+    host,
+    handle = None,
+    silent = None,
+    redirect = None,
+    timeout = None
+):
     is_relative = location.startswith("/")
     if is_relative: location = scheme + "://" + host + location
     return get(
         location,
         handle = handle,
-        redirect = redirect
+        silent = silent,
+        redirect = redirect,
+        timeout = timeout
     )
 
 def _resolve(*args, **kwargs):
+    # obtains the reference to the global set of variables, so
+    # that it's possible to obtain the proper resolver method
+    # according to the requested client
     _global = globals()
+
+    # tries to retrieve the global configuration values that
+    # will condition the way the request is going to be performed
     client = config.conf("HTTP_CLIENT", "netius")
+    reuse = config.conf("HTTP_REUSE", True, cast = bool)
+
+    # tries to determine the set of configurations requested on
+    # a request basis (not global) these have priority when
+    # compared with the global configuration ones
     client = kwargs.pop("client", client)
+    reuse = kwargs.pop("reuse", reuse)
+
+    # sets the value for connection re-usage so that a connection
+    # pool is used if request, otherwise one connection per request
+    # is going to be used (at the expense of resources)
+    kwargs["reuse"] = reuse
+
+    # tries to retrieve the reference to the resolve method for the
+    # current client and then runs it, retrieve then the final result,
+    # note that the result structure may be engine dependent
     resolver = _global.get("_resolve_" + client, _resolve_legacy)
     try: result = resolver(*args, **kwargs)
     except ImportError: result = _resolve_legacy(*args, **kwargs)
     return result
 
-def _resolve_legacy(url, method, headers, data, timeout, **kwargs):
+def _resolve_legacy(url, method, headers, data, silent, timeout, **kwargs):
     opener = legacy.build_opener(legacy.HTTPHandler)
     request = legacy.Request(url, data = data, headers = headers)
     request.get_method = lambda: method
     return opener.open(request, timeout = timeout)
 
-def _resolve_netius(url, method, headers, data, timeout, **kwargs):
+def _resolve_requests(url, method, headers, data, silent, timeout, **kwargs):
+    util.import_pip("requests")
+    import requests
+    global _requests_session
+
+    # retrieves the various dynamic parameters for the http client
+    # usage under the requests infra-structure
+    reuse = kwargs.get("reuse", True)
+    connections = kwargs.get("connections", 256)
+
+    # verifies if the session for the requests infra-structure is
+    # already created and if that's not the case and the re-use
+    # flag is sets creates a new session for the requested settings
+    registered = "_requests_session" in globals()
+    if not registered and reuse:
+        _requests_session = requests.Session()
+        adapter = requests.adapters.HTTPAdapter(
+            pool_connections = connections,
+            pool_maxsize = connections
+        )
+        _requests_session.mount("", adapter)
+
+    # determines the based object from which the concrete methods
+    # are going to be loaded by inspecting the re-use flag
+    if reuse: base = _requests_session
+    else: base = requests
+
+    # converts the string based method value into a lower cased value
+    # and then uses it to retrieve the method object method (callable)
+    # that is going to be called to perform the request
+    method = method.lower()
+    caller = getattr(base, method)
+
+    # runs the caller method (according to selected method) and waits for
+    # the result object converting it then to the target response object
+    result = caller(url, headers = headers, data = data, timeout = timeout)
+    response = HTTPResponse(
+        data = result.content,
+        code = result.status_code,
+        headers = result.headers
+    )
+
+    # retrieves the response code of the created response and verifies if
+    # it represent an error, if that's the case raised an error exception
+    # to the upper layers to break the current execution logic properly
+    code = response.getcode()
+    is_error = _is_error(code)
+    if is_error: raise legacy.HTTPError(
+        url, code, "HTTP retrieval problem", None, response
+    )
+
+    # returns the final response object to the caller method, this object
+    # should comply with the proper upper layers structure
+    return response
+
+def _resolve_netius(url, method, headers, data, silent, timeout, **kwargs):
+    util.import_pip("netius")
     import netius.clients
 
+    # determines the final value of the silent flag taking into
+    # account if the current infra-structure is not running under
+    # a development environment
+    silent = silent or False
+    silent |= not common.is_devel()
+
     # converts the provided dictionary of headers into a new map to
     # allow any re-writing of values, valuable for a re-connect
     headers = dict(headers)
 
+    # tries to determine the proper level of verbosity to be used by
+    # the client, for that the system tries to determine if the current
+    # execution environment is a development one (verbose)
+    level = logging.CRITICAL if silent else logging.DEBUG
+
     # retrieves the various dynamic parameters for the http client
     # usage under the netius infra-structure
     retry = kwargs.get("retry", 1)
     reuse = kwargs.get("reuse", True)
+    level = kwargs.get("level", level)
+    async = kwargs.get("async", False)
+    callback = kwargs.get("callback", None)
+
+    # re-calculates the retry and re-use flags taking into account
+    # the async flag, if the execution mode is async we don't want
+    # to re-use the http client as it would create issues
+    retry, reuse = (0, False) if async else (retry, reuse)
+
+    # creates the proper set of extra parameters to be sent to the
+    # http client taking into account a possible async method request
+    extra = _async_netius(callback) if async else dict()
 
     # verifies if client re-usage must be enforced and if that's the
     # case the global client object is requested (singleton) otherwise
     # the client should be created inside the http client static method
-    http_client = _client_netius() if reuse else None
+    http_client = _client_netius(level = level) if reuse else None
     result = netius.clients.HTTPClient.method_s(
         method,
         url,
         headers = headers,
         data = data,
-        async = False,
-        http_client = http_client
+        async = async,
+        timeout = timeout,
+        http_client = http_client,
+        level = level,
+        **extra
     )
 
+    # if the async mode is defined an invalid value is returned immediately
+    # as the processing will be taking place latter (on callback)
+    if async: return None
+
     # tries to retrieve any possible error coming from the result object
     # if this happens it means an exception has been raised internally and
     # the error should be handled in a proper manner, if the error is related
     # to a closed connection a retry may be performed to try to re-establish
     # the connection (allows for reconnection in connection pool)
     error = result.get("error", None)
-    if error == "closed":
-        if retry < 1: raise exceptions.OperationalError(message = "Connection closed")
+    if error == "closed" and retry > 0:
         kwargs["retry"] = retry - 1
         return _resolve_netius(
-            url, method, headers, data, timeout, **kwargs
+            url, method, headers, data, silent, timeout, **kwargs
         )
 
     # converts the netius specific result map into a response compatible
     # object (equivalent to the urllib one) to be used by the upper layers
-    # under an equivalent and compatible approach
+    # under an equivalent and compatible approach note that this conversion
+    # may raise an exception in case the result represent an error
     response = netius.clients.HTTPClient.to_response(result)
 
     # retrieves the response code of the created response and verifies if
     # it represent an error, if that's the case raised an error exception
     # to the upper layers to break the current execution logic properly
     code = response.getcode()
-    is_error = code // 100 in (4, 5) if code else True
+    is_error = _is_error(code)
     if is_error: raise legacy.HTTPError(
         url, code, "HTTP retrieval problem", None, response
     )
@@ -503,22 +766,69 @@ def _resolve_netius(url, method, headers, data, timeout, **kwargs):
     # may be used freely under the compatibility interface it provides
     return response
 
-def _client_netius():
+def _client_netius(level = logging.CRITICAL):
     import netius.clients
     global _netius_clients
-    registered = "_netius_clients" in globals()
-    _netius_clients = _netius_clients if registered else dict()
+
+    # retrieves the reference to the current thread and uses the value
+    # to retrieve the thread identifier (tid) for it, to be used in the
+    # identification of the client resource associated with it
     tid = threading.current_thread().ident
-    netius_client = _netius_clients.get(tid, None)
+
+    # acquires the global http lock and executes a series of validation
+    # and initialization of the netius client infra-structure, this
+    # operations required thread safety
+    ACCESS_LOCK.acquire()
+    try:
+        registered = "_netius_clients" in globals()
+        _netius_clients = _netius_clients if registered else dict()
+        netius_client = _netius_clients.get(tid, None)
+    finally:
+        ACCESS_LOCK.release()
+
+    # in case a previously created netius client has been retrieved
+    # returns it to the caller method for proper re-usage
     if netius_client: return netius_client
+
+    # creates the "new" http client for the current thread and registers
+    # it under the netius client structure so that it may be re-used
     netius_client = netius.clients.HTTPClient(
         thread = False,
-        auto_pause = True
+        auto_pause = True,
+        level = level
     )
     _netius_clients[tid] = netius_client
+
+    # in case this is the first registration of the dictionary a new on
+    # exit callback is registered to cleanup the netius infra-structure
+    # then the final client is returned to the caller of the method
     if not registered: common.base().on_exit(_cleanup_netius)
     return netius_client
 
+def _async_netius(callback):
+    import netius.clients
+
+    buffer = []
+    extra = dict()
+
+    def _on_close(connection):
+        callback and callback(None)
+
+    def _on_data(client, parser, data):
+        data = data
+        data and buffer.append(data)
+
+    def _callback(connection, parser, message):
+        result = netius.clients.HTTPClient.set_request(parser, buffer)
+        response = netius.clients.HTTPClient.to_response(result)
+        callback and callback(response)
+
+    extra["callback"] = _callback
+    extra["on_close"] = _on_close
+    extra["on_data"] = _on_data
+
+    return extra
+
 def _cleanup_netius():
     global _netius_clients
     for netius_client in _netius_clients.values(): netius_client.cleanup()
@@ -729,10 +1039,8 @@ def _try_boundary(fields, boundary, doseq = False):
         values = values if is_list else [values]
 
         for value in values:
-            value_t = type(value)
-
-            if value_t == dict: name = ""; value = value.get("data", b"")
-            elif value_t == tuple: name = value[0]; value = value[1]
+            if isinstance(value, dict): name = ""; value = value.get("data", b"")
+            elif isinstance(value, tuple): name = value[0] or ""; value = value[1]
             else: name = ""; value = _encode(value)
 
             if not key.find(boundary) == -1: return False
@@ -741,8 +1049,39 @@ def _try_boundary(fields, boundary, doseq = False):
 
     return True
 
+def _is_error(code):
+    return code // 100 in (4, 5) if code else True
+
 def _encode(value, encoding = "utf-8"):
     value_t = type(value)
     if value_t == legacy.BYTES: return value
     elif value_t == legacy.UNICODE: return value.encode(encoding)
     return legacy.bytes(str(value))
+
+class HTTPResponse(object):
+    """
+    Compatibility object to be used by HTTP libraries that do
+    not support the legacy HTTP response object as a return
+    for any of their structures.
+    """
+
+    def __init__(self, data = None, code = 200, status = None, headers = None):
+        self.data = data
+        self.code = code
+        self.status = status
+        self.headers = headers
+
+    def read(self):
+        return self.data
+
+    def readline(self):
+        return self.read()
+
+    def close(self):
+        pass
+
+    def getcode(self):
+        return self.code
+
+    def info(self):
+        return self.headers
