@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2016 Hive Solutions Lda.
+# Copyright (c) 2008-2017 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2016 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -219,26 +219,35 @@ def chri(value):
     if type(value) in INTEGERS: return _chr(value)
     return value
 
-def bytes(value, encoding = "latin-1"):
-    if not PYTHON_3: return value
+def bytes(value, encoding = "latin-1", errors = "strict", force = False):
+    if not PYTHON_3 and not force: return value
     if value == None: return value
     if type(value) == _bytes: return value
-    return value.encode(encoding)
+    return value.encode(encoding, errors)
 
-def str(value, encoding = "latin-1"):
-    if not PYTHON_3: return value
+def str(value, encoding = "latin-1", errors = "strict", force = False):
+    if not PYTHON_3 and not force: return value
     if value == None: return value
-    if type(value) == _str: return value
-    return value.decode(encoding)
+    if type(value) in STRINGS: return value
+    return value.decode(encoding, errors)
+
+def u(value, encoding = "utf-8", errors = "strict", force = False):
+    if PYTHON_3 and not force: return value
+    if value == None: return value
+    if type(value) == UNICODE: return value
+    return value.decode(encoding, errors)
+
+def ascii(value, encoding = "utf-8", errors = "replace"):
+    if is_bytes(value): value = value.decode(encoding, errors)
+    else: value = UNICODE(value)
+    value = value.encode("ascii", errors)
+    value = str(value)
+    return value
 
 def orderable(value):
     if not PYTHON_3: return value
     return Orderable(value)
 
-def u(value, encoding = "utf-8"):
-    if PYTHON_3: return value
-    return value.decode(encoding)
-
 def is_str(value):
     return type(value) == _str
 
@@ -267,7 +276,7 @@ def execfile(path, global_vars, local_vars = None, encoding = "utf-8"):
     finally: file.close()
     data = data.decode(encoding)
     code = compile(data, path, "exec")
-    exec(code, global_vars, local_vars)
+    exec(code, global_vars, local_vars) #@UndefinedVariable
 
 def walk(path, visit, arg):
     for root, dirs, _files in os.walk(path):
