@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2016 Hive Solutions Lda.
+# Copyright (c) 2008-2017 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2016 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -41,18 +41,24 @@ import inspect
 import logging
 import threading
 
-LOGGING_FORMAT = "%(asctime)s [%(levelname)s] %(message)s"
+from . import config
+
+LOGGING_FORMAT = "%%(asctime)s [%%(levelname)s] %s%%(message)s"
 """ The format to be used for the logging operation in
 the app, these operations are going to be handled by
 multiple stream handlers """
 
-LOGGING_FORMAT_TID = "%(asctime)s [%(levelname)s] [%(thread)d] %(message)s"
+LOGGING_FORMAT_TID = "%%(asctime)s [%%(levelname)s] %s[%%(thread)d] %%(message)s"
 """ The format to be used for the logging operation in
 the app, these operations are going to be handled by
 multiple stream handlers, this version of the string
 includes the thread identification number and should be
 used for messages called from outside the main thread """
 
+LOGGING_EXTRA = "[%(name)s] " if config.conf("LOGGING_EXTRA", cast = bool) else ""
+""" The extra logging attributes that are going to be applied
+to the format strings to obtain the final on the logging """
+
 MAX_LENGTH = 10000
 """ The maximum amount of messages that are kept in
 memory until they are discarded, avoid a very large
@@ -85,6 +91,9 @@ LEVEL_ALIAS = {
 """ Map defining a series of alias that may be used latter
 for proper debug level resolution """
 
+LOGGING_FORMAT = LOGGING_FORMAT % LOGGING_EXTRA
+LOGGING_FORMAT_TID = LOGGING_FORMAT_TID % LOGGING_EXTRA
+
 class MemoryHandler(logging.Handler):
     """
     Logging handler that is used to store information in
@@ -98,7 +107,12 @@ class MemoryHandler(logging.Handler):
         self.messages = []
         self.messages_l = {}
 
-        formatter = ThreadFormatter(LOGGING_FORMAT)
+        format = config.conf("LOGGING_FORMAT", None)
+        format_base = format or LOGGING_FORMAT
+        format_tid = format or LOGGING_FORMAT_TID
+
+        formatter = ThreadFormatter(format_base)
+        formatter.set_tid(format_tid)
         self.setFormatter(formatter)
 
     def get_messages_l(self, level):
@@ -171,16 +185,22 @@ class ThreadFormatter(logging.Formatter):
     the threading printing the log records is not the main one.
     """
 
+    def __init__(self, *args, **kwargs):
+        logging.Formatter.__init__(self, *args, **kwargs)
+        self._tidfmt = logging.Formatter(self._fmt)
+
     def format(self, record):
         # retrieves the reference to the current thread and verifies
         # if it represent the current process main thread, then selects
         # the appropriate formating string taking that into account
         current = threading.current_thread()
         is_main = current.name == "MainThread"
-        if is_main: self._fmt = LOGGING_FORMAT
-        else: self._fmt = LOGGING_FORMAT_TID
+        if not is_main: return self._tidfmt.format(record)
         return logging.Formatter.format(self, record)
 
+    def set_tid(self, value):
+        self._tidfmt = logging.Formatter(value)
+
 def rotating_handler(
     path = "appier.log",
     max_bytes = 1048576,
