@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2016 Hive Solutions Lda.
+# Copyright (c) 2008-2017 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2016 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -90,8 +90,9 @@ an extension to the base builder map """
 METAS = dict(
     text = lambda v, d: v,
     enum = lambda v, d: d["enum"].get(v, None),
-    list = lambda v, d: json.dumps(v),
-    map = lambda v, d: json.dumps(v),
+    list = lambda v, d: json.dumps(v, ensure_ascii = False),
+    map = lambda v, d: json.dumps(v, ensure_ascii = False),
+    longmap = lambda v, d: json.dumps(v, ensure_ascii = False),
     date = lambda v, d: datetime.datetime.utcfromtimestamp(float(v)).strftime("%d %b %Y"),
     datetime = lambda v, d: datetime.datetime.utcfromtimestamp(float(v)).strftime("%d %b %Y %H:%M:%S")
 )
@@ -222,7 +223,7 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
     Abstract model class from which all the models should
     directly or indirectly inherit. Should provide the
     basic infra-structure for the persistence of data using
-    a play key value storage engine.
+    a plain key value storage engine.
 
     The data should always be store in a dictionary oriented
     structure while it's not persisted in the database.
@@ -579,14 +580,14 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
             end = skip + limit,
             size = size,
             total = total,
-            sorter = request.params_s.get("sorter", None),
-            direction = request.params_s.get("direction", "descending")
+            sorter = request.params_f.get("sorter", None),
+            direction = request.params_f.get("direction", "descending")
         )
 
         def generate(**kwargs):
             # creates a copy of the current definition of the parameters and for each
             # of the exclusion parameters removes it from the current structure
-            params = dict(request.params_s)
+            params = dict(request.params_f)
             if "async" in params: del params["async"]
 
             # retrieves the "special" sorter keyword based argument and the equivalent
@@ -875,6 +876,11 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         definition = cls.definition()
         return definition.get(name, {})
 
+    @classmethod
+    def register(cls, lazy = False):
+        if lazy: return
+        cls.setup()
+
     @classmethod
     def setup(cls):
         indexes = cls.indexes()
@@ -1787,7 +1793,15 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         validate = True,
         is_new = None,
         increment_a = None,
-        immutables_a = None
+        immutables_a = None,
+        pre_validate = True,
+        pre_save = True,
+        pre_create = True,
+        pre_update = True,
+        post_validate = True,
+        post_save = True,
+        post_create = True,
+        post_update = True
     ):
         # checks if the instance to be saved is a new instance
         # or if this is an update operation and then determines
@@ -1800,13 +1814,16 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         # should ensure that the model is ready to be saved in the
         # data source, without corruption of it, only run this process
         # in case the validate flag is correctly set
-        validate and self._validate()
+        validate and self._validate(
+            pre_validate = pre_validate,
+            post_validate = post_validate
+        )
 
         # calls the complete set of event handlers for the current
         # save operation, this should trigger changes in the model
-        self.pre_save()
-        is_new and self.pre_create()
-        not is_new and self.pre_update()
+        pre_save and self.pre_save()
+        pre_create and is_new and self.pre_create()
+        pre_update and not is_new and self.pre_update()
 
         # filters the values that are present in the current model
         # so that only the valid ones are stored in, invalid values
@@ -1833,18 +1850,18 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
 
         # calls the post save event handlers in order to be able to
         # execute appropriate post operations
-        self.post_save()
-        is_new and self.post_create()
-        not is_new and self.post_update()
+        post_save and self.post_save()
+        post_create and is_new and self.post_create()
+        post_update and not is_new and self.post_update()
 
         # returns the instance that has just been used for the save
         # operation, this may be used for chaining operations
         return self
 
-    def delete(self):
+    def delete(self, pre_delete = True, post_delete = True):
         # calls the complete set of event handlers for the current
         # delete operation, this should trigger changes in the model
-        self.pre_delete()
+        pre_delete and self.pre_delete()
 
         # retrieves the reference to the store object to be able to
         # execute the removal command for the current model
@@ -1857,7 +1874,7 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
 
         # calls the complete set of event handlers for the current
         # delete operation, this should trigger changes in the model
-        self.post_delete()
+        post_delete and self.post_delete()
 
     def approve(self, model = None, type = None):
         # retrieves the class associated with the instance
@@ -1951,10 +1968,16 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
     def _delete(self):
         pass
 
-    def _validate(self, model = None, method = None):
+    def _validate(
+        self,
+        model = None,
+        method = None,
+        pre_validate = True,
+        post_validate = True
+    ):
         # calls the event handler for the validation process this
         # should setup the operations for a correct validation
-        self.pre_validate()
+        pre_validate and self.pre_validate()
 
         # starts the model reference with the current model in
         # case none is defined
@@ -2009,7 +2032,7 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
 
         # calls the event handler for the validation process this
         # should finish the operations from a correct validation
-        self.post_validate()
+        post_validate and self.post_validate()
 
     def _filter(
         self,
@@ -2165,7 +2188,7 @@ class Action(dict):
     interface in order to provide backwards compatibility.
     """
 
-    def cast(self, values):
+    def cast(self, values, keyword = False):
         """
         Runs the "casting" operation for a series of provided
         values, the order sequence of the provided values is
@@ -2175,29 +2198,35 @@ class Action(dict):
         :type values: List
         :param values: The sequence containing the various values
         that are going to be casted according to the operation spec.
-        :rtype: List
+        :type keyword: bool
+        :param keyword: If a keyword based dictionary of values
+        should be created instead for dynamic binding.
+        :rtype: List/Dictionary
         :return: The sequence of values now casted with the proper
-        data types as specified in operation structure.
+        data types as specified in operation structure or alternatively
+        the same values under a named dictionary.
         """
 
-        # creates the list that will hold the "final" casted values
+        # creates the list/map that will hold the "final" casted values
         # should have the same size of the input values list
-        casted = []
+        casted = {} if keyword else []
 
         # retrieves the reference to the parameters specification
         # and then uses it in the iteration for the casting of the
         # various "passed" raw values
         parameters = self.get("parameters", [])
         for value, parameters in zip(values, parameters):
+            name = parameters[1]
             type = parameters[2]
             cast = type
             is_default = value in (None, "")
             cast = BUILDERS.get(cast, cast)
             if cast and not is_default: value = cast(value)
             if is_default: value = TYPE_DEFAULTS.get(type, value)
-            casted.append(value)
+            if keyword: casted[name] = value
+            else: casted.append(value)
 
-        # returns the final list of casted values to the caller method
+        # returns the final list/map of casted values to the caller method
         # so that it may be used safely in the context
         return casted
 
