@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2016 Hive Solutions Lda.
+# Copyright (c) 2008-2017 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2016 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -141,9 +141,13 @@ class Request(object):
         self.code = 200
         self.location = prefix + util.quote(path).lstrip("/")
         self.content_type = None
+        self.cache_control = None
         self.authorization = None
         self.data = None
         self.result = None
+        self.locale = None
+        self.query_s = None
+        self.prefixes = None
         self.session = session.MockSession(self)
         self.set_cookie = None
         self.post = {}
@@ -154,6 +158,7 @@ class Request(object):
         self.out_headers = {}
         self.warnings = []
         self.properties = {}
+        self._closed = False
         self._params = None
 
     def __str__(self):
@@ -162,6 +167,53 @@ class Request(object):
     def __unicode__(self):
         return self.__str__()
 
+    def close(self):
+        """
+        Called upon the closing of the request so that the internal references
+        the request contains may be related (avoiding memory leaks).
+
+        After a request is closed it cannot be re-opened and a new request must
+        be created instead if a new one is required.
+        """
+
+        self.owner = None
+        self.method = None
+        self.path = None
+        self.prefix = None
+        self.query = None
+        self.scheme = None
+        self.address = None
+        self.params = None
+        self.data_j = None
+        self.environ = None
+        self.session_c = None
+        self.handled = None
+        self.context = None
+        self.method_i = None
+        self.json = None
+        self.code = None
+        self.location = None
+        self.content_type = None
+        self.cache_control = None
+        self.authorization = None
+        self.data = None
+        self.result = None
+        self.locale = None
+        self.query_s = None
+        self.prefixes = None
+        self.session = None
+        self.set_cookie = None
+        self.post = None
+        self.files = None
+        self.args = None
+        self.cookies = None
+        self.in_headers = None
+        self.out_headers = None
+        self.warnings = None
+        self.properties = None
+        self._closed = True
+        self._params = None
+
     def handle(self, code = None, result = ""):
         """
         Handles the current request, setting the response code
@@ -225,7 +277,8 @@ class Request(object):
             prefix = self.prefix,
             query = self.query,
             scheme = self.scheme,
-            content_type = self.content_type
+            content_type = self.content_type,
+            cache_conrtol = self.cache_control
         )
 
     def warning(self, message):
@@ -299,12 +352,26 @@ class Request(object):
 
     def default_content_type(self, default):
         if self.content_type: return
+        if self.has_header_out("Content-Type"): return
         self.content_type = default
 
-    def get_header(self, name, default = None):
+    def get_cache_control(self):
+        return self.cache_control
+
+    def set_cache_control(self, cache_control):
+        self.cache_control = cache_control
+
+    def default_cache_control(self, default):
+        if self.cache_control: return
+        if self.has_header_out("Cache-Control"): return
+        self.cache_control = default
+
+    def get_header(self, name, default = None, normalize = True):
+        if normalize: name = name.title()
         return self.in_headers.get(name, default)
 
-    def set_header(self, name, value):
+    def set_header(self, name, value, normalize = False):
+        if normalize: name = name.title()
         self.out_headers[name] = str(value)
 
     def set_headers(self, headers):
@@ -312,6 +379,14 @@ class Request(object):
         for name, value in headers.items():
             self.set_header(name, value)
 
+    def has_header_in(self, name, insensitive = True):
+        if insensitive: name = name.title()
+        return name in self.in_headers
+
+    def has_header_out(self, name, insensitive = True):
+        if insensitive: name = name.title()
+        return name in self.out_headers
+
     def get_address(self, resolve = True):
         """
         Retrieves the client (network) address associated with the
@@ -329,11 +404,20 @@ class Request(object):
         """
 
         if not resolve: return self.address
-        return self.get_header("X-Forwarded-For", self.address)
+        address = self.get_header("X-Forwarded-For", self.address)
+        address = self.get_header("X-Client-IP", address)
+        address = self.get_header("X-Real-IP", address)
+        address = address.split(",", 1)[0].strip()
+        return address
 
     def resolve_params(self):
         self.params = self._resolve_p(self.params)
 
+    def resolve_query_s(self, prefixes = ("x-",)):
+        parts = [part for part in self.query.split("&") if not part.startswith(prefixes)]
+        self.query_s = "&".join(parts)
+        self.prefixes = prefixes
+
     def load_base(self):
         self.load_data()
         self.load_form()
@@ -569,6 +653,10 @@ class Request(object):
         user_agent = self.get_header("User-Agent", None)
         return util.is_mobile(user_agent)
 
+    def is_tablet(self):
+        user_agent = self.get_header("User-Agent", None)
+        return util.is_tablet(user_agent)
+
     def is_success(self):
         return self.code == 200
 
@@ -576,14 +664,30 @@ class Request(object):
         return self.code in (204, 304)
 
     @property
-    def location_f(self):
-        if not self.query: return self.location
-        return self.location + "?" + self.query
+    def location_f(self, safe = True):
+        query = self.query_s if safe else self.query
+        if not query: return self.location
+        return self.location + "?" + query
+
+    @property
+    def params_f(self, safe = True):
+        if not safe: return self.params_s
+        if hasattr(self, "_params_f"): return self._params_f
+        self._params_f = dict([(key, value) for key, value in self.params_s.items() if\
+            not key.startswith(self.prefixes)])
+        return self._params_f
+
+    @property
+    def async(self):
+        return True if self.get_header("X-Async") else False
+
+    @property
+    def partial(self):
+        return True if self.get_header("X-Partial") else False
 
     def _resolve_p(self, params):
         secret = self.session.get("secret", None)
         if not secret: return params
-
         raise exceptions.AppierException(message = "Not implemented")
 
 class MockRequest(Request):
@@ -599,3 +703,6 @@ class MockRequest(Request):
         self.files_s = dict()
         self.post_s = dict()
         self.params_s = dict()
+
+    def close(self):
+        pass
