@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2016 Hive Solutions Lda.
+# Copyright (c) 2008-2017 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2016 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -101,6 +101,19 @@ class UtilTest(unittest.TestCase):
         result = appier.date_to_timestamp("1984-13-29", format = "%Y-%m-%d")
         self.assertEqual(result, None)
 
+    def test_camel_to_underscore(self):
+        result = appier.camel_to_underscore("HelloWorld")
+        self.assertEqual(type(result), str)
+        self.assertEqual(result, "hello_world")
+
+        result = appier.camel_to_underscore("HELLOWorld")
+        self.assertEqual(type(result), str)
+        self.assertEqual(result, "hello_world")
+
+        result = appier.camel_to_underscore("HELLOWorldHELLOWorld")
+        self.assertEqual(type(result), str)
+        self.assertEqual(result, "hello_world_hello_world")
+
     def test_dict_merge(self):
         first = dict(a = "hello", b = "world")
         second = dict(a = "hello_new", b = "world_new", c = "other")
