@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2016 Hive Solutions Lda.
+# Copyright (c) 2008-2017 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,21 +31,21 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2016 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
 """ The license for the module """
 
-import os
+import uuid
 import base64
 import hashlib
-import tempfile
 
 from . import util
 from . import crypt
 from . import legacy
 from . import common
+from . import storage
 
 class Type(object):
 
@@ -82,6 +82,9 @@ class File(Type):
         hash = file_m.get("hash", None)
         mime = file_m.get("mime", None)
         etag = file_m.get("etag", None)
+        guid = file_m.get("guid", None)
+        params = file_m.get("params", None)
+        engine = file_m.get("engine", None)
 
         is_valid = name and data_b64
         data_b64_b = legacy.bytes(data_b64)
@@ -90,6 +93,7 @@ class File(Type):
         size = len(data) if is_valid else 0
         hash = hash or self._hash(data)
         etag = etag or self._etag(data)
+        guid = guid or self._guid()
 
         self.data = data
         self.data_b64 = data_b64
@@ -99,6 +103,11 @@ class File(Type):
         self.file_name = name
         self.mime = mime
         self.etag = etag
+        self.guid = guid
+        self.params = params
+        self.engine = engine
+
+        self._load()
 
     def build_t(self, file_t):
         name, content_type, data = file_t
@@ -109,6 +118,7 @@ class File(Type):
         size = len(data) if is_valid else 0
         etag = self._etag(data)
         hash = self._hash(data)
+        guid = self._guid()
 
         self.data = data
         self.data_b64 = data_b64
@@ -118,6 +128,11 @@ class File(Type):
         self.file_name = name
         self.mime = content_type
         self.etag = etag
+        self.guid = guid
+        self.params = None
+        self.engine = None
+
+        self._load()
 
     def build_i(self, file):
         self.data = file.data
@@ -128,6 +143,11 @@ class File(Type):
         self.file_name = file.file_name
         self.mime = file.mime
         self.etag = file.etag
+        self.guid = file.guid
+        self.params = file.params
+        self.engine = file.engine
+
+        self._load()
 
     def build_f(self, file):
         self.data = None
@@ -138,20 +158,51 @@ class File(Type):
         self.file_name = file.filename
         self.mime = file.content_type
         self.etag = None
+        self.guid = self._guid()
+        self.params = None
+        self.engine = None
 
-        self._flush()
+        self._load()
 
-    def read(self):
-        return self.data
+    def read(self, size = None):
+        engine = self._engine()
+        return engine.read(self, size = size)
+
+    def seek(self, offset = None):
+        engine = self._engine()
+        return engine.seek(self, offset = offset)
+
+    def delete(self):
+        engine = self._engine()
+        return engine.delete(self)
+
+    def cleanup(self):
+        engine = self._engine()
+        return engine.cleanup(self)
 
     def json_v(self, *args, **kwargs):
+        if not self.is_valid(): return None
+        store = kwargs.get("store", True)
+        if store: self._store()
+        data = self.data_b64 if self.is_stored() else None
         return dict(
             name = self.file_name,
-            data = self.data_b64,
+            data = data,
             hash = self.hash,
             mime = self.mime,
-            etag = self.etag
-        ) if self.is_valid() else None
+            etag = self.etag,
+            guid = self.guid,
+            params = self.params,
+            engine = self.engine
+        )
+
+    def is_seekable(self):
+        engine = self._engine()
+        return engine.is_seekable()
+
+    def is_stored(self):
+        engine = self._engine()
+        return engine.is_stored()
 
     def is_valid(self):
         return self.file_name or (self.data or self.data_b64)
@@ -171,24 +222,36 @@ class File(Type):
         digest = hash.hexdigest()
         return digest
 
-    def _flush(self):
-        if not self.file_name: return
-        if self.data: return
+    def _guid(self):
+        return str(uuid.uuid4())
 
-        path = tempfile.mkdtemp()
-        path_f = os.path.join(path, self.file_name)
-        self.file.save(path_f)
+    def _load(self, force = False):
+        engine = self._engine()
+        engine.load(self, force = force)
 
-        file = open(path_f, "rb")
-        try: data = file.read()
-        finally: file.close()
+    def _store(self, force = False):
+        engine = self._engine()
+        engine.store(self, force = force)
 
-        self.data = data
-        self.data_b64 = base64.b64encode(data)
+    def _compute(self):
+        """
+        Computes a series of "calculated" attributes related with
+        the data associated with the current file, these values may
+        include: length, hash values, etag, etc.
+
+        This method should be called whenever the data attributes are
+        changed as defined in specification.
+        """
+
+        self.data_b64 = base64.b64encode(self.data)
         self.data_b64 = legacy.str(self.data_b64)
-        self.hash = self._hash(data)
-        self.size = len(data)
-        self.etag = self._etag(data)
+        self.hash = self._hash(self.data)
+        self.size = len(self.data)
+        self.etag = self._etag(self.data)
+
+    def _engine(self):
+        if not self.engine: return storage.BaseEngine
+        return getattr(storage, self.engine.capitalize() + "Engine")
 
 class Files(Type):
 
@@ -229,8 +292,8 @@ class Files(Type):
     def is_empty(self):
         return len(self._files) == 0
 
-    def _flush(self):
-        for file in self._files: file._flush()
+    def _load(self):
+        for file in self._files: file._load()
 
 class ImageFile(File):
 
@@ -257,16 +320,14 @@ class ImageFile(File):
         self._ensure_all()
 
     def json_v(self, *args, **kwargs):
-        return dict(
-            name = self.file_name,
-            data = self.data_b64,
-            hash = self.hash,
-            mime = self.mime,
-            etag = self.etag,
+        if not self.is_valid(): return None
+        value = File.json_v(self, *args, **kwargs)
+        value.update(
             width = self.width,
             height = self.height,
             format = self.format
-        ) if self.is_valid() else None
+        )
+        return value
 
     def _ensure_all(self):
         self._ensure_size()
