@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (c) 2008-2016 Hive Solutions Lda.
+# Copyright (c) 2008-2017 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2016 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -65,6 +65,15 @@ CREATION_COUNTER = 0
 will be used to create an order in the declaration
 of attributes for a set of classes """
 
+FIRST_CAP_REGEX = re.compile("(.)([A-Z][a-z]+)")
+""" Regular expression that ensures that the first
+token of each camel string is properly capitalized """
+
+ALL_CAP_REGEX = re.compile("([a-z0-9])([A-Z])")
+""" The generalized transition from lower case to
+upper case letter regex that will provide a way of
+putting the underscore in the middle of the transition """
+
 SORT_MAP = {
     "1" : 1,
     "-1" : -1,
@@ -175,6 +184,27 @@ def is_mobile(user_agent):
     is_mobile = True if mobile or mobile_prefix else False
     return is_mobile
 
+def is_tablet(user_agent):
+    """
+    Verifies if the provided user agent string represent a
+    tablet agent, for that a series of regular expressions
+    are matched against the user agent string.
+
+    :type user_agent: String
+    :param user_agent: The string containing the user agent
+    value that is going to be verified against a series of
+    regular expressions for tablet verification.
+    :rtype: bool
+    :return: If the provided user agent string represents a
+    tablet browser or a regular (desktop) one.
+    """
+
+    prefix = user_agent[:4]
+    tablet = defines.TABLET_REGEX.search(user_agent)
+    mobile_prefix = defines.MOBILE_PREFIX_REGEX.search(prefix)
+    is_tablet = True if tablet or mobile_prefix else False
+    return is_tablet
+
 def email_parts(base, encoding = None):
     """
     Unpacks the complete set of parts (name and email) from the
@@ -244,6 +274,46 @@ def obfuscate(value, display_l = 3, token = "*"):
     obfuscated = value[:display_l] + ((value_l - display_l) * token)
     return obfuscated
 
+def import_pip(name, package = None, default = None):
+    package = package or name
+    try: module = __import__(name)
+    except:
+        try: module = install_pip_s(package)
+        except: return default
+        try: module = __import__(name)
+        except: return default
+    return module
+
+def ensure_pip(name, package = None, async = True):
+    package = package or name
+    try:
+        __import__(name)
+    except:
+        install_pip_s(package, async = async)
+
+def install_pip(package, async = False, user = False):
+    import pip
+    args = ["install", package]
+    if user: args.insert(1, "--user")
+    if async:
+        import multiprocessing
+        process = multiprocessing.Process(
+            target = pip.main,
+            args = (args,)
+        )
+        process.start()
+        result = 0
+    else:
+        result = pip.main(args)
+    if result == 0: return
+    raise exceptions.OperationalError(message = "pip error")
+
+def install_pip_s(package, async = False):
+    try:
+        install_pip(package, async = async, user = False)
+    except exceptions.OperationalError:
+        install_pip(package, async = async, user = True)
+
 def request_json(request = None, encoding = "utf-8"):
     # retrieves the proper request object, either the provided
     # request or the default base request object and then in
@@ -490,12 +560,12 @@ def leafs(object):
     # to the caller method so that it may be used there
     return leafs_l
 
-def gen_token(limit = None):
+def gen_token(limit = None, hash = hashlib.sha256):
     """
     Generates a random cryptographic ready token according
     to the framework specification, this is generated using
     a truly random uuid based seed and hashed using the
-    sha256 hash digest.
+    provided hash digest strategy (sha256 by default).
 
     The resulting value is returned as an hexadecimal based
     string according to the standard.
@@ -503,13 +573,17 @@ def gen_token(limit = None):
     :type limit: int
     :param limit: The maximum number of characters allowed
     for the token to be generated.
+    :type hash: Function
+    :param hash: The hashing method that is going to be used
+    for the hash of the generated token, this should be compliant
+    with the base python hashing infra-structure.
     :rtype: String
     :return: The hexadecimal based string value
     """
 
     token_s = str(uuid.uuid4())
     token_s = token_s.encode("utf-8")
-    token = hashlib.sha256(token_s).hexdigest()
+    token = hash(token_s).hexdigest()
     if limit: token = token[:limit]
     return token
 
@@ -553,7 +627,7 @@ def html_to_text(data):
     data = data.replace("\n.", ".")
     return data
 
-def camel_to_underscore(camel):
+def camel_to_underscore(camel, separator = "_"):
     """
     Converts the provided camel cased based value into
     a normalized underscore based string.
@@ -564,22 +638,18 @@ def camel_to_underscore(camel):
     :type camel: String
     :param camel: The camel cased string that is going to be
     converted into an underscore based string.
+    :type separator: String
+    :param separator: The separator token that is going to
+    be used in the camel to underscore conversion.
     :rtype: String
-    :return The underscore based string resulting from the
+    :return: The underscore based string resulting from the
     conversion of the provided camel cased one.
     """
 
-    values = []
-    camel_l = len(camel)
-
-    for index in range(camel_l):
-        char = camel[index]
-        is_upper = char.isupper()
-
-        if is_upper and not index == 0: values.append("_")
-        values.append(char)
-
-    return "".join(values).lower()
+    value = FIRST_CAP_REGEX.sub(r"\1" + separator + r"\2", camel)
+    value = ALL_CAP_REGEX.sub(r"\1" + separator + r"\2", value)
+    value = value.lower()
+    return value
 
 def camel_to_readable(camel):
     """
@@ -647,6 +717,48 @@ def unquote(value, *args, **kwargs):
     if is_bytes: value = value.decode("utf-8")
     return value
 
+def call_safe(callable, *args, **kwargs):
+    """
+    Method used to call a callable object using a "safe" approach,
+    meaning that each of its keyword arguments will be validated
+    for existence in the target callable definition.
+
+    In case the validation of the keyword argument fails the same
+    argument is removed from the map of keyword arguments.
+
+    Note that in case the wildcard based kwargs value exists in
+    the callable definition the callable is immediately considered
+    to be valid and the call is ran.
+
+    :type callable: Callable
+    :callable callable: The callable that is going to have the keyword
+    based arguments validated and the get called.
+    :rtype: object
+    :return: The resulting value from the safe call of the provided
+    callable, this may have any data type.
+    """
+
+    # retrieves the arguments specification to the provided callable
+    # and retrieves the various argument names and the existence or
+    # not of the wildcard kwargs value in the callable and in case it
+    # exists runs the callable call immediately
+    argspec = legacy.getargspec(callable)
+    method_args = argspec[0]
+    method_kwargs = argspec[2]
+    if method_kwargs: return callable(*args, **kwargs)
+
+    # iterates over the complete set of keyword based arguments to be
+    # used in the call and validates them against the method specification
+    # in case they do not exist in the specification deletes them from
+    # the map of keyword based arguments (not going to be sent)
+    for name in legacy.keys(kwargs):
+        if name in method_args: continue
+        del kwargs[name]
+
+    # runs the callable with the "remaining" arguments and keyword arguments
+    # returning the value to the caller method
+    return callable(*args, **kwargs)
+
 def base_name(name, suffix = "_controller"):
     """
     Retrieves the base name of a class name that contains
@@ -751,6 +863,7 @@ def parse_multipart(data, boundary):
     post = dict()
     files = dict()
 
+    boundary = str(boundary)
     boundary = boundary.strip()
     boundary_base = "--" + boundary[9:].strip("\"")
     boundary_value = legacy.bytes(boundary_base + "\r\n")
@@ -1008,6 +1121,7 @@ def private(function):
 def ensure(token = None):
 
     def decorator(function):
+
         @functools.wraps(function)
         def interceptor(self, *args, **kwargs):
             ensure = kwargs.get("ensure", True)
@@ -1210,6 +1324,14 @@ def ctx_locale(name = "", force = False):
     try: yield locale.setlocale(locale.LC_ALL, name)
     finally: locale.setlocale(locale.LC_ALL, saved)
 
+@contextlib.contextmanager
+def ctx_request(app = None):
+    app = app or common.base().get_app()
+    _request = app._request
+    app._request = app._mock
+    try: yield True
+    finally: app._request = _request
+
 class FileTuple(tuple):
     """
     Tuple class (inherits from tuple) that represents
